//Similar to extractFile.ts, but instead of receiving and returning an ArrayBuffer, works with Node.js streams.

import * as stream from 'stream';
import * as zlib from 'zlib';
import { ISsnFileEntry } from '../interfaces/ISsnFileEntry';
import streamSetMaxLength from './streams/streamSetMaxLength';

/** Extracts the file with the given metadata from the stream.
 * The stream must already start at the .zip's local file header
 * and must transparently span across multiple disks if necessary.
 */
export default function extractFileStream(file: ISsnFileEntry, inputStream: stream.Readable): stream.Readable {
  const localFileHeader = new DataView(inputStream.read(30));

  //Local file header signature
  if (localFileHeader.getUint32(0, true) !== 0x04034B50) {
    throw new Error('Local file header had wrong magic');
  }
  //All fields in the local file header are copies of the central file header, so we can skip them.
  //FIXME: Maybe we should actually read these fields to verify that they are identical?
  //skip 22 bytes
  const localFilenameSize = localFileHeader.getUint16(26, true);
  const localExtraSize = localFileHeader.getUint16(28, true);

  //skip local file name and extra field
  inputStream.read(localFilenameSize + localExtraSize);

  //TODO: pipe into decryption if file is encrypted
  const decryptTransform = new stream.Transform({
    read() {
      //...
    },
  });
  inputStream.pipe(decryptTransform);

  //TODO: pipe into decompression if file is compressed
  const decompressTransform = zlib.createInflateRaw();
  decryptTransform.pipe(decompressTransform);

  //TODO: output file
  return streamSetMaxLength(decompressTransform, file.size);

  /*const out = new stream.Readable();

  return out;*/
}
