import * as stream from 'stream';

/** Takes the given ReadableStream and returns a ReadableStream with the same contents but that terminates after the given length. */
export default function streamSetMaxLength(inputStream: stream.Readable, maxLength: number): stream.Readable {
  if (maxLength <= 0) {
    throw new Error('maxLength is out of bounds.');
  }

  let remaining = maxLength;

  const outStream = new stream.Readable({
    encoding: 'binary',
    read(size) {
      //If no size is provided, just pass through all remaining bytes
      if (size === undefined) {
        this.push(inputStream.read(remaining));
        remaining = 0;
        //End is reached, terminate stream
        this.push(null);
      } else {
        //Otherwise, pass through however many bytes we can
        const clampedSize = Math.min(size, remaining);
        this.push(inputStream.read(clampedSize));
        remaining -= clampedSize;

        //If end is reached, terminate stream
        if (remaining <= 0) {
          this.push(null);
        }
      }
    },
  });

  return outStream;
}
