import * as stream from 'stream';
import { TextDecoder } from 'util';

const decoder = new TextDecoder('utf-8');

export default function streamToString(inputStream: stream.Readable): Promise<string> {
  return new Promise((resolve, reject) => {
    let outputString = '';

    //Convert chunks to string
    inputStream.on('data', (chunk: Buffer) => {
      outputString += decoder.decode(chunk, { stream: true });
    });

    //Output final string
    inputStream.on('end', () => {
      outputString += decoder.decode();
      resolve(outputString);
    });

    //Exit on error
    inputStream.on('error', (error) => {
      reject(error);
    });
  });
}
